//-------------------------------------------------------------
//UART0.h : header file for the ENS34 UART0
//-------------------------------------------------------------

#ifndef _UART_H_
#define _UART_H_

#include <stdio.h>

#include "MyTypeDefs.h"

//-------------------------------------------
// Definition of global includes
//-------------------------------------------

//-------------------------------------------
// Definition of global constants
//-------------------------------------------

#define UART1     // define usage of UART0 or UART1
//#define FARROM    // define usage of FAR-makros to access strings above 64 kByte (e.g. AtMega128-Bootloader)
//#define PRINTF  // define usage of printf()
//#define TRACE   // define if you want trace debug info

//-------------------------------------------
// Declaration of global variables
//-------------------------------------------

//-------------------------------------------
// Declaration of global macros
//-------------------------------------------

// Defines for UART Baudrates
#define  BR_4800    (u16)(F_CPU / 8 /   4800 - 0.5)
#define  BR_9600    (u16)(F_CPU / 8 /   9600 - 0.5)
#define  BR_14400   (u16)(F_CPU / 8 /  14400 - 0.5)
#define  BR_19200   (u16)(F_CPU / 8 /  19200 - 0.5)
#define  BR_28800   (u16)(F_CPU / 8 /  28800 - 0.5)
#define  BR_38400   (u16)(F_CPU / 8 /  38400 - 0.5)
#define  BR_57600   (u16)(F_CPU / 8 /  57600 - 0.5)
#define  BR_76800   (u16)(F_CPU / 8 /  76800 - 0.5)
#define  BR_115200  (u16)(F_CPU / 8 / 115200 - 0.5)
#define  BR_234000  (u16)(F_CPU / 8 / 234000 - 0.5)

#ifdef UART0

#include "UART0.h"

#ifdef FARROM
  #define UART_puts_I(pgmemstr)   UART0_puts_PF(FAR(*PSTR(pgmemstr))) // put immediate far string to TX-Buffer (wait if full)
  #define UART_putl_I(pgmemstr)   UART0_putl_PF(FAR(*PSTR(pgmemstr))) // put immediate far string to TX-Buffer (wait if full)
#else
  #define UART_puts_I(pgmemstr)   UART0_puts_P(PSTR(pgmemstr))        // put immediate string to TX-Buffer (wait if full)
  #define UART_putl_I(pgmemstr)   UART0_putl_P(PSTR(pgmemstr))        // put immediate string + CRLF to TX-Buffer (wait if full)
#endif  //#ifdef FARROM

#ifdef PRINTF
  #ifdef FARROM
    #define printf_I(pgmemstr)      printf_P(FAR(*PSTR(pgmemstr)))
  #else
    #define printf_I(pgmemstr)      printf_P(PSTR(pgmemstr))
  #endif  //#ifdef FARROM
  #define UART_putchar            UART0_putchar
#endif  //#ifdef PRINTF

#ifdef TRACE
  #define TRACES(tracestr)      UART0_traces(tracestr)           // put timestamp + string to TX-Buffer (wait if full)
  #define TRACEL(tracestr)      UART0_tracel(tracestr)           // put timestamp + string + [CR+LF] to TX-Buffer (wait if full)
  #define TRACEM                UART0_tracem()                   // put '-------------------------------------------------' + [CR+LF]
  #define TRACES_I(pgmemstr)    UART0_traces_P(PSTR(pgmemstr))   // put timestamp + pgmemstring  to TX-Buffer (wait if full)
  #define TRACEL_I(pgmemstr)    UART0_tracel_P(PSTR(pgmemstr))   // put timestamp + pgmemstring + [CR+LF] to TX-Buffer (wait if full)
#else
  #define TRACES(pgmemstr)
  #define TRACEL(pgmemstr)
  #define TRACES_I(pgmemstr)
  #define TRACEL_I(pgmemstr)
#endif

#define UART_echo(param)        UART0_echo(param)
#define UART_init(param)        UART0_init(param)
#define UART_getl(param)        UART0_getl(param)
#define UART_putc(param)        UART0_putc(param)
#define UART_puts(param)        UART0_puts(param)

#endif  //#ifdef UART0

#ifdef UART1

#include "UART1.h"


#ifdef FARROM
  #define UART_puts_I(pgmemstr)   UART1_puts_PF(FAR(*PSTR(pgmemstr))) // put immediate far string to TX-Buffer (wait if full)
  #define UART_putl_I(pgmemstr)   UART1_putl_PF(FAR(*PSTR(pgmemstr))) // put immediate far string to TX-Buffer (wait if full)
#else
  #define UART_puts_I(pgmemstr)   UART1_puts_P(PSTR(pgmemstr))        // put immediate string to TX-Buffer (wait if full)
  #define UART_putl_I(pgmemstr)   UART1_putl_P(PSTR(pgmemstr))        // put immediate string + CRLF to TX-Buffer (wait if full)
#endif  //#ifdef FARROM

#ifdef PRINTF
  #ifdef FARROM
    #define printf_I(pgmemstr)      printf_P(FAR(*PSTR(pgmemstr)))
  #else
    #define printf_I(pgmemstr)      printf_P(PSTR(pgmemstr))
  #endif  //#ifdef FARROM
  #define UART_putchar            UART1_putchar
#endif  //#ifdef PRINTF

#ifdef TRACE
  #define TRACES(tracestr)      UART1_traces(tracestr)            // put timestamp + string to TX-Buffer (wait if full)
  #define TRACEL(tracestr)      UART1_tracel(tracestr)            // put timestamp + string + [CR+LF] to TX-Buffer (wait if full)
  #define TRACEM                UART1_tracem()                    // put '-------------------------------------------------' + [CR+LF]
  #define TRACES_I(pgmemstr)    UART1_traces_P(PSTR(pgmemstr))    // put timestamp + pgmemstring  to TX-Buffer (wait if full)
  #define TRACEL_I(pgmemstr)    UART1_tracel_P(PSTR(pgmemstr))    // put timestamp + pgmemstring + [CR+LF] to TX-Buffer (wait if full)
#else
  #define TRACES(pgmemstr)
  #define TRACEL(pgmemstr)
  #define TRACES_P(pgmemstr)
  #define TRACEL_P(pgmemstr)
#endif

#define UART_echo(param)        UART1_echo(param)
#define UART_init(param)        UART1_init(param)
#define UART_getl(param)        UART1_getl(param)
#define UART_putc(param)        UART1_putc(param)
#define UART_puts(param)        UART1_puts(param)

#endif  //#ifdef UART1

#endif  // _UART_H_

