/*
    AtMega644P Bootlader for
    Shutdown Management Box - Net for IFASS-Mehrzwecksender
 * ----------------------------------------------------------------------------
    Bootsector start at 0xE000 (=> Word-Address 0x7000)
 * ----------------------------------------------------------------------------
  Programm Fuse Bits (L-Fuse = 0xB7, H-Fuse = 0xD0, E-Fuse = 0xFD)
   Brown out detection enabled @ 2.7V
   Bootreset vector enabled @ 0x7000 (Word Address)
   Boot FLASH size = 4096 Words
   CKSEL: Full Swwing Oszillator, Delay = 16 kCykles + 64 ms)
   CKDIV8 = No;
   CKOUT  = Yes;
   EESAV  = Yes;
   JTAGEN = No;
   OCDEN  = No;
   SPIEN  = Yes,
   WDTON  = No
 * ----------------------------------------------------------------------------
 IDE: Eclipse with WinAvr
  Compiler: winavr-20080610
  Compilersettings:
    Device: atmega644P
    Frequency: 20 MHz
    Optimisation: -O1
    ISO C99 + GNU-Extensions -std=gnu99
        (x) Unsigned Chars
        (x) Unsigned Bitfields
        (x) Pack Structure Members
        (x) Short Enums

  Linker: --section-start=.text=0xE000

  Changes:
  --> V1.0   28.08.2009 psa: First Version derived from COT400A Bootloader
  --> V1.1   01.09.2009 psa: Booting can be stopped by pressing [Enter] during startup

 */

//-------------------------------------------
// Definition of local includes
//-------------------------------------------

#include <avr/eeprom.h>
#include "boot.h"
#include "main.h"
#include "UART.h"
#include "Command.h"


//-------------------------------------------
// Definition of local constants
//-------------------------------------------


//-------------------------------------------
// Declaration of global variables
//-------------------------------------------

u08  MSG[MAX_MSG_LEN];            // global buffer for command message
u08  RSP[MAX_MSG_LEN];            // global buffer for response message
u08  LEN;                         // length of command message
u08  UPDATE=0;                    // flags for App-update status info (Passed between APP => BLDR)

//-------------------------------------------
// Declaration of local functions
//-------------------------------------------

void AVR_init(void);             // init AVR uC
u08 BOOT_check(u08 status);      // check reason for reset

//--------------------------------------------------------------------------------
// Main
//--------------------------------------------------------------------------------


int main(void)
{
  u08 status;
  u16 temp;
  //----------------------------------------------------------------------
  // psa 17.08.2006 Don't change this entry-code block
  //----------------------------------------------------------------------
  status = MCUSR;                    // save MCUC(S)R reset status at very first
  MCUSR = status & 0xE0;             // then clear reset status, before touching the watchdog
  wdt_disable();                     // now, disable watchdog (important for avr6)
  MCUCR = (1<<IVCE);                 // enable interruptvectors change
  MCUCR = (1<<IVSEL);                // move interruptvectors to the Boot-Sector
  boot_lock_bits_set(1<<BLB11);      // protect bootloader-section
  //----------------------------------------------------------------------
  // disable dangerous interrupts
  //----------------------------------------------------------------------
  TIMSK0 = 0;
  TIMSK1 = 0;
  TIMSK2 = 0;
  TWCR   = 0;
  SPCR   = 0;
  UCSR0B = 0;
  UCSR1B = 0;
  EIMSK  = 0;
  PCICR  = 0;
  ACSR   = 0;
  ADCSRA = 0;
  //----------------------------------------------------------------------
  AVR_init();
  sei();
  //----------------------------------------------------------------------
  status=BOOT_check(status);            // check reset reason
  _delay_ms(100);                       // wait some time for UART-Output
  if (status)                           // for normal reset reason
  {
    temp=pgm_read_word(0x0000);         // get opcode for first instruction in application
    if (temp!=0xFFFF)                   // if not 0xFFFF (application is programmed)
    {
      _delay_ms(400);                   // wait to check for a [CR]
      if (!(LEN=UART_getl((char*)MSG))) // and if no [CR] has ben read
      {
        cli();
        asm("jmp 0x000000");            // then Jump to application
      }
    }
  }
  UART_putl_I("=> booting aborted!");
  UART_putc('>');
  UART_echo(ON);
  //----------------------------------------------------------------------
  while(1)
  {
    if ((LEN=UART_getl((char*)MSG)))
    {
      cmd_check();
      UART_putc('>');
    }
    wdt_reset();
  }
}

//--------------------------------------------------------------------------------
// Functions
//--------------------------------------------------------------------------------


void AVR_init(void)
{
  u32 baudrate;
  u16 baud;
  eeprom_read_block(&baudrate, EEPTR_BAUD, 4);
  baud = check_baudrate(&baudrate);
  if (baud)
  {
    UART_init(baud);
  }
  else
  {
    UART_init(BR_19200);
  }
  wdt_enable(WDTO_2S);          // enable watchdog to 2 second timeout
  //---------------------------------------
}


u08 BOOT_check(u08 status)
{
  //----------------------------------------------------------------------
  // Startup Messages
  //----------------------------------------------------------------------
  if (status & (1<<BORF))                         // Brown-Out Reset
  {
    UART_putl_I("\r\nBrown-Out Reset");
  }
  else if (status & (1<<PORF))                    // Power-On Reset
  {
    UART_putl_I("\r\nPower-On Reset");
  }
  else if (status & (1<<JTRF))                    // JTAG Reset
  {
    UART_putl_I("\r\nJTAG Reset");
  }
  else if (status & (1<<EXTRF))                   // External Reset
  {
    UART_putl_I("\r\nExternal Reset");
  }
  else if (status & (1<<WDRF))                    // Watchdog Reset
  {
    UART_putl_I("\r\nWatchdog Reset");
  }
  else                                            // no reset
  {
    UART_putl_I("=> Bootloader Jump");
    status=0;
  }
  //----------------------------------------------------------------------
  UART_putl_I("Bootloader V-1.1");
  UART_puts_I("Press [Enter] to abort booting: ");
  return(status);
}

