//-------------------------------------------
//main.h : header file for the Bootloader project
//-------------------------------------------

#ifndef _MAIN_H_
#define _MAIN_H_

//-------------------------------------------
// Definition of global includes
//-------------------------------------------

// AVR-LibC specific
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include <avr/boot.h>


// GCC specific
#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>

// ENS34 Project specific
#include "tools.h"

//-------------------------------------------
// Definition of global constants
//-------------------------------------------
#define ON   (u08)0x01
#define OFF  (u08)0x00

// Status of FW-Upgrade
#define UPDATE_DONE   0x00
#define UPDATE_NOAPP  0x01
#define UPDATE_PROGS  0x02
#define UPDATE_READY  0x04
#define UPDATE_ERROR  0x80

#define MAX_MSG_LEN   128

#define  EEPTR_BAUD  ((void *)0x0010) // Baudrate

// MCU EEPROM-Addresse Pointers
#define  RIEXP  5                     // exponent for invenventory string size 2^[5] = 32 Byte
#define  RIBASE 0x0100                // base address for remote inventory strings
#define  RISIZE (1<<RIEXP)            // size for remote inventory strings (32 Byte)
#define  RIA    (RIBASE+RISIZE*00)    // Remote InventoryString 'A'	Board name [16]
#define  RIB    (RIBASE+RISIZE*01)    // Remote InventoryString 'B'	TDHS part number [16]
#define  RIC    (RIBASE+RISIZE*02)    // Remote InventoryString 'C'	TDHS serial number [16]
#define  RID    (RIBASE+RISIZE*03)    // Remote InventoryString 'D'	Date of manufacturing/ First test and Tester ID [16]
#define  RIE    (RIBASE+RISIZE*04)    // Remote InventoryString 'E'	Date of last Calibration/ Test and tester ID [16]
#define  RIF    (RIBASE+RISIZE*05)    // Remote InventoryString 'F'	Date of last HW-modification/ Repair and Tester ID [16]
#define  RIG    (RIBASE+RISIZE*06)    // Remote InventoryString 'G'	HW-revision of Board [16]
#define  RIH    (RIBASE+RISIZE*07)    // Remote InventoryString 'H'	Comment of this Board (1st line) [16]
#define  RII    (RIBASE+RISIZE*08)    // Remote InventoryString 'I'	Comment of this Board (2nd line) [16]
#define  RIJ    (RIBASE+RISIZE*09)    // Remote InventoryString 'J'
#define  RIK    (RIBASE+RISIZE*10)    // Remote InventoryString 'K'	Comment of this Board (3rd line) [16]
#define  RIL    (RIBASE+RISIZE*11)    // Remote InventoryString 'L'	Comment of this Board (4th line) [16]
#define  RIM    (RIBASE+RISIZE*12)    // Remote InventoryString 'M'
#define  RIN    (RIBASE+RISIZE*13)    // Remote InventoryString 'N'
#define  RIO    (RIBASE+RISIZE*14)    // Remote InventoryString 'O'
#define  RIP    (RIBASE+RISIZE*15)    // Remote InventoryString 'P'	SW revision of Boot loader [16]
#define  RIQ    (RIBASE+RISIZE*16)    // Remote InventoryString 'Q'	SW revision of Application [16]
#define  RIR    (RIBASE+RISIZE*17)    // Remote InventoryString 'R'
#define  RIS    (RIBASE+RISIZE*18)    // Remote InventoryString 'S'
#define  RIT    (RIBASE+RISIZE*19)    // Remote InventoryString 'T'
#define  RIU    (RIBASE+RISIZE*20)    // Remote InventoryString 'U'
#define  RIV    (RIBASE+RISIZE*21)    // Remote InventoryString 'V'
#define  RIW    (RIBASE+RISIZE*22)    // Remote InventoryString 'W'
#define  RIX    (RIBASE+RISIZE*23)    // Remote InventoryString 'X'
#define  RIY    (RIBASE+RISIZE*24)    // Remote InventoryString 'Y'
#define  RIZ    (RIBASE+RISIZE*25)    // Remote InventoryString 'Z'	Date of last SW download [16]

//-------------------------------------------
// Declaration of global const+variables
//-------------------------------------------

extern prog_char BOARD[];       // String-const for Board-name
extern prog_char FW_REVISION[]; // String-const for Firmware Rev.
extern u08 MSG[];               // global buffer for command message
extern u08 RSP[];               // global buffer for response message
extern u08 LEN;                 // length of command message
extern u08 COM;                 // command switch for UART0 r UART1
extern u08 UPDATE;              // is true while FW-update is pending
extern u16 BL_timer;            // delay timer for automatically jump to app

//-------------------------------------------
// Declaration of global functions
//-------------------------------------------

void (*jump_to_app)(void);       // APPL_RESET_VECTOR	=		0x0000
void (*jump_to_bl)(void);        // BOOT_RESET_VECTOR			0xF000

#endif
