//--------------------------------------------------------
//tools.c : source file for several utilities
//--------------------------------------------------------

//-------------------------------------------
// Definition of local includes
//-------------------------------------------

#include <avr/eeprom.h>
#include "MyTypeDefs.h"
#include "tools.h"
#include "main.h"
//#include "timer.h"

//-------------------------------------------
// Definition of local constants
//-------------------------------------------

//-------------------------------------------
// Declaration of local variables
//-------------------------------------------

//static u08 ascii[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

//-------------------------------------------
// Declaration of local functions
//-------------------------------------------

//-------------------------------------------
// Code for functions
//-------------------------------------------

//u16 ATOMIC16(volatile u16 *var)
////-------------------------------------------
//// avoid ISR during access to the 2 bytes of a
//// volatile int, which might be modified in ISR
//// => Atomic volatile access oustide of ISR
////-------------------------------------------
//{
//  u16 temp;
//  cli();         // disable interrupts
//  temp=*var;     // get value of volatile var
//  sei();         // enable interrupts
//  return(temp);  // return the value
//}
//
//u32 ATOMIC32(volatile u32 *var)
////---------------------------------------------
//// avoid ISR during access to the 4 bytes of a
//// volatile long, which might be modified in ISR
//// => Atomic volatile access oustide of ISR
////---------------------------------------------
//{
//  u32 temp;
//  cli();         // disable interrupts
//  temp=*var;     // get value of volatile var
//  sei();         // enable interrupts
//  return(temp);  // return the value
//}
//
//void EEPROM_ws(u16 addr, u08 *str, u08 n)
////---------------------------------------------------------------------------
//// write string with n Bytes (including stringend '/0') to EEPROM[addr]
//// If the string is longer than n, it will be truncated to n.
////---------------------------------------------------------------------------
//{
//  u08 i=0, chr=0;
//  n--;                                    // n-1 for '/0' char at stringend
//  while(i<n)                              // while not stringend
//  {
//    chr=str[i++];                         // get next char
//    eeprom_write_byte((u08*)addr++,chr);  // and write char to eeprom
//    if (chr==0) i=n;                      // if stringend, then exit loop
//  }
//  if (chr)                                // if last char was not the stringend
//  {
//    eeprom_write_byte((u08*)addr,0x00);   // then append stringend in eeprom
//  }
//}
//
//u08* EEPROM_rs(u16 addr, u08 *str, u08 n)
////---------------------------------------------------------------------------
//// read string with n Bytes (including stringend '/0') from EEPROM[addr]
//// If the string is longer than n, it will be truncated to n.
////---------------------------------------------------------------------------
//{
//  u08 i=0, chr=0;
//  n--;                                    // n-1 for '/0' char at stringend
//  while(i<n)                              // while not stringend
//  {
//    chr=eeprom_read_byte((u08*)addr++);   // read char from eeprom
//    if (chr==0xFF)                        // replace 0xFF by stringend
//    {
//      chr=0x00;
//    }
//    str[i++]=chr;                         // and add it to string
//    if (chr==0) i=n;                      // if stringend, then exit loop
//  }
//  if (chr)                                // if last char was not the stringend
//  {
//    str[i]=0x00;                          // then append stringend to string
//  }
//  return(str);
//}
//
//
//u08* _int2dez(u32 *intval, u08 *dezstr, u08 n)
////---------------------------------------------------------------------
//// Converts an integer with a size of [n] bytes to a dezimal string.
////---------------------------------------------------------------------
//{
//  if ((n>2) && (*intval & 0xFFFF0000))
//  {
//    ultoa(*intval,(char *)dezstr,10);
//  }
//  else if (n>1)
//  {
//    utoa((u16)*intval,(char *)dezstr,10);
//  }
//  else
//  {
//    utoa((u08)*intval,(char *)dezstr,10);
//  }
//  return(dezstr);
//}
//
//void _dez2int(u08 *dezstr, u32 *intval)
////---------------------------------------------------------------------
//// Converts a dezimal string to an integer
////---------------------------------------------------------------------
//{
//  *intval= atol((char *)dezstr);
//}
//
//
//u08* _int2hex(u08 *intval, u08 *hexstr, u08 n)
////---------------------------------------------------------------------
//// Converts an integer with a size of [n] bytes to a hex-digit string.
//// The string will be filled up with leading zeros to his length.
////---------------------------------------------------------------------
//{
//  u08 e=0;
//  while(n--)
//  {
//    hexstr[e++]=ascii[(intval[n] >> 0x04)];
//    hexstr[e++]=ascii[(intval[n] &  0x0F)];
//  }
//  hexstr[e]=0x00;
//  return(hexstr);
//}
//
//u08* _bin2hex(u08 *intval, u08 *hexstr, u08 n)
////---------------------------------------------------------------------
//// Converts a sequence of [n] bytes to a hex-digit string.
////---------------------------------------------------------------------
//{
//  u08 e=0,i=0;
//  while(i<n)
//  {
//    hexstr[e++]=ascii[(intval[i] >> 0x04)];
//    hexstr[e++]=ascii[(intval[i] &  0x0F)];
//    i++;
//  }
//  hexstr[e]=0x00;
//  return(hexstr);
//}


void _hex2int(u08 *hexstr, u08 *intval, u08 n)
//---------------------------------------------------------------------
// Converts a hex-digit string to an integer with a size of [n] bytes.
// If the string doesn't contain enought hex-digits, the string will
// be modified by right alignment and insertion of leading zeros.
//---------------------------------------------------------------------
{
  u08 chr,e,i=0;
  //-------------------------------------------------------------------
  // check stringlength and adjust it, if necessery
  //-------------------------------------------------------------------
	e=n<<1;                           // e=2*n (two hex-digits per byte)
  while((i<e) && (hexstr[i])) i++;  // scan for early stringend
  if (i<e)                          // if early stringend found
	{
    hexstr[e]=0x00;                 // set new stringend
    while(e--)                      // loop for each char
    {
      if (i) hexstr[e]=hexstr[--i]; // if hex digit then move to right aligned position
      else   hexstr[e]='0';         // else insert leading '0'
    }
  }
  //-------------------------------------------------------------------
  // Convert string to binary integer
  //-------------------------------------------------------------------
  i=0; e=0;
  while(n--)                        // for each byte
  {
    chr = (hexstr[i++] | 0x20);     // get high-char and mask it to lowercase
    if (chr<'a') e = (chr-48);      // if '0'..'9' => value = (chr + 0 - '0')
    else e = (chr-87);              // if 'a'..'f' => value = (chr +10 - 'a')
    e = e<<4;                       // nibble shift (char = char * 16)
    chr = (hexstr[i++] | 0x20);     // get low-char and mask it to lowercase
    if (chr<'a') e |= (chr-48);     // if '0'..'9' => value |= (chr + 0 - '0')
    else e |= (chr-87);             // if 'a'..'f' => value |= (chr +10 - 'a')
    intval[n] = e;                  // store binary byte value
  }
}

u08 _hex2bin(u08 *hexstr, u08 *intval, u08 n)
//---------------------------------------------------------------------
// Converts a hex-digit string to a sequenze of [n] bytes.
// if [n] is zero, the sequenze will be limited by the stringlength
// The checksum of the sequence is provided as return value.
//---------------------------------------------------------------------
{
  u08 chr, cs=0, hex=0, e=0, i=0;
  if (n==0) n=120;                  // if n=0 then set maximum stringlength to 2*120 char
  chr=hexstr[0];                    // get first char
  while((e<n) && (chr))             // while not stringend
  {
    chr |= 0x20;                    // mask char to lowercase
    if (chr<'a') hex |= (chr-48);   // if '0'..'9' => value = (chr + 0 - '0')
    else hex |= (chr-87);           // if 'a'..'f' => value = (chr +10 - 'a')
    if (i & 1)                      // if low nibble (second nibble)
    {
      cs = cs+hex;                  // calculate checksum
      intval[e++]=hex;              // store binary byte value
      hex=0;                        // clear binary value for the next byte
    }
    else hex = hex<<4;              // else nibble shift (char = char * 16)
    chr=hexstr[++i];                // get next char
  }
  return(cs);                       // return checksum
}
