//-----------------------------------------
//tools.h : header file several utilities
//-----------------------------------------

#ifndef _TOOLS_H_
#define _TOOLS_H_

//-------------------------------------------
// Definition of global includes
//-------------------------------------------

#include "MyTypeDefs.h"

//-------------------------------------------
// Definition of global constants
//-------------------------------------------

//-------------------------------------------
// Declaration of global variables
//-------------------------------------------

//-------------------------------------------
// Declaration of global functions
//-------------------------------------------

//--------------------------------------------------
// => Atomic 16 bit volatile access oustide of ISR
//--------------------------------------------------
u16 ATOMIC16(volatile u16 *var);

//--------------------------------------------------
// => Atomic 32 bit volatile access oustide of ISR
//--------------------------------------------------
u32 ATOMIC32(volatile u32 *var);

//---------------------------------------------------------------------------
// read string with n Bytes (including stringend '/0') from EEPROM[addr]
// If the string is longer than n, it will be truncated to n.
//---------------------------------------------------------------------------
u08* EEPROM_rs(u16 addr, u08 *str, u08 n);

//---------------------------------------------------------------------------
// write string with n Bytes (including stringend '/0') to EEPROM[addr]
// If the string is longer than n, it will be truncated to n.
//---------------------------------------------------------------------------
void EEPROM_ws(u16 addr, u08 *str, u08 n);

//---------------------------------------------------------------------------
// Analogue to strcpy() and strcpy_P(): strcpy_PF() is usable for
// PROGMEM-strings which are located above 64kB (far)
//---------------------------------------------------------------------------
#define strcpy_PF(str,pgmemstr) (_strcpy_PF(str,FAR(pgmemstr)))
u08* _strcpy_PF(u08 *str, u32 pgm_ptr);

//---------------------------------------------------------------------
// Converts an integer with a size of [n] bytes to a dezimal string.
// If the string will be filled up with leading zeros to his length.
//---------------------------------------------------------------------
#define int2dez(val,str) _int2dez((u32*)&(val),(str),sizeof((val)))
u08* _int2dez(u32 *intval, u08 *dezstr, u08 n);

//---------------------------------------------------------------------
// Converts a dezimal string to an integer
//---------------------------------------------------------------------
#define dez2int(str,val) _dez2int((str),(u32*)&(val))
void _dez2int(u08 *dezstr, u32 *intval);

//--------------------------------------------------------------------
// Converts an integer with a size of [n] bytes to a hex-digit string.
// With macro for simplified _int2str() function call
//--------------------------------------------------------------------
#define int2hex(val,str) _int2hex((u08*)&(val),(str),sizeof((val)))
u08* _int2hex(u08 *intval, u08 *hexstr, u08 n);

//--------------------------------------------------------------------
// Converts a hex-digit string to an integer with a size of [n] bytes.
// With macro for simplified _str2int() function call
//--------------------------------------------------------------------
#define hex2int(str,val) _hex2int((str),(u08*)&(val),sizeof((val)))
void _hex2int(u08 *hexstr, u08 *intval, u08 n);

//---------------------------------------------------------------------
// Converts a sequence of [n] bytes to a hex-digit string.
//---------------------------------------------------------------------
#define bin2hex(val,str,len) _bin2hex((u08*)&(val),(str),(len))
u08* _bin2hex(u08 *intval, u08 *hexstr, u08 n);

//---------------------------------------------------------------------
// Converts a hex-digit string to a sequenze of [n] bytes.
// if [n] is zero, the sequenze will be limited by the stringlength
// The checksum of the sequence is provided as return value.
//---------------------------------------------------------------------
#define hex2bin(str,val,len) _hex2bin((str),(u08*)&(val),(len))
u08 _hex2bin(u08 *hexstr, u08 *intval, u08 n);


#endif
